package de.intevation.flys.artifacts.model.sq;

import org.apache.log4j.Logger;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;
import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DataFacet;
import de.intevation.flys.artifacts.model.FacetTypes;

import de.intevation.flys.artifacts.states.DefaultState.ComputeType;


/**
 * Facet to show the curve in a sq relation.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public class SQOutlierFacet extends DataFacet implements FacetTypes {

    private static final Logger log = Logger.getLogger(SQOutlierFacet.class);

    public static final int BITMASK_ITERATION = (1 << 16) - 1;


    private int fractionIdx;


    public SQOutlierFacet() {
    }


    public SQOutlierFacet(
        int    idx,
        int    fractionIdx,
        String name,
        String description,
        String hash,
        String stateId
    ) {
        super(idx, name, description, ComputeType.ADVANCE, hash, stateId);
        this.fractionIdx = fractionIdx;
    }


    @Override
    public Object getData(Artifact artifact, CallContext context) {
        log.debug("SQOutlierFacet.getData");

        if (artifact instanceof FLYSArtifact) {
            FLYSArtifact flys = (FLYSArtifact) artifact;

            CalculationResult res = (CalculationResult) flys.compute(
                context, ComputeType.ADVANCE, false);

            int idx  = this.index >> 16;
            int iter = this.index & BITMASK_ITERATION;

            if (log.isDebugEnabled()) {
                log.debug("Fetch data for index : " + this.index);
                log.debug("  > index:       " + idx);
                log.debug("  > fraction:    " + fractionIdx);
                log.debug("  > iteration:   " + iter);
            }

            SQResult[]       result  = (SQResult[]) res.getData();
            SQFractionResult fResult = result[idx].getFraction(fractionIdx);

            if (fResult == null) {
                log.warn("No SQFractionResult at " + idx + "|" + fractionIdx);
            }
            else if (log.isDebugEnabled()) {
                SQ[] outliers = fResult.getOutliers(iter);
                int  num      = outliers != null ? outliers.length : 0;

                log.debug("Found " + num + " outliers for iteration " + iter);
            }

            return fResult.getOutliers(iter);
        }

        return null;
    }


    @Override
    public SQOutlierFacet deepCopy() {
        SQOutlierFacet copy = new SQOutlierFacet();
        copy.set(this);

        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
